#include "HDC1000_Defs.h"

sbit sRDY at GPIOD_IDR.B11;
char tmp_data[12];

float temperature;
float humidity;
float dewpoint;

static void sensor_setup() {
  tmp_data[0] = 0x02;
  tmp_data[1] = 0x10;
  tmp_data[2] = 0x00;
  I2C1_Start();
  I2C1_Write(0x40,tmp_data,3,END_MODE_STOP);
  delay_ms(5);
}

//Conversion time for 14bit values of temperature and humidity 6,5ms + 6,35ms
static void wait_conversion() {
  delay_ms(20);
}

volatile unsigned int temp_value,humidity_value;

void Read_Sensor() {
  tmp_data[0] = TEMP_I2C_ADDR;
  I2C1_Start();
  I2C1_Write(0x40,tmp_data,1,END_MODE_RESTART);
  wait_conversion();
  I2C1_Read(HDC1000_I2C_ADDR,tmp_data,4,END_MODE_STOP);
  temp_value = (tmp_data[0] << 8) + tmp_data[1];
  humidity_value = (tmp_data[2] << 8) + tmp_data[3];
  temperature = ((float)temp_value * Tconstant)-40;
  humidity = (float)humidity_value * Hconstant;
}

void Read_Temperature() {
  unsigned int temp_value;
  tmp_data[0] = TEMP_I2C_ADDR;
  I2C1_Start();
  I2C1_Write(0x40,tmp_data,1,END_MODE_RESTART);
  delay_ms(7);
  I2C1_Read(HDC1000_I2C_ADDR,tmp_data,2,END_MODE_STOP);
  temp_value = (tmp_data[0] << 8) + tmp_data[1];
  temperature = ((float)temp_value * Tconstant)-40;
}

void Read_Humidity() {
  unsigned int humidity_value;
  tmp_data[0] = TEMP_I2C_ADDR;
  I2C1_Start();
  I2C1_Write(0x40,tmp_data,1,END_MODE_RESTART);
  delay_ms(7);
  I2C1_Read(HDC1000_I2C_ADDR,tmp_data,2,END_MODE_STOP);
  humidity_value = (tmp_data[2] << 8) + tmp_data[3];
  humidity = (float)humidity_value * Hconstant;
}

unsigned int Read_DeviceID() {
  unsigned int id_value;
  tmp_data[0] = DEVICEID_I2C_ADDR;
  I2C1_Start();
  I2C1_Write(0x40,tmp_data,1,END_MODE_RESTART);
  I2C1_Read(HDC1000_I2C_ADDR,tmp_data,2,END_MODE_STOP);
  id_value = (tmp_data[0] << 8) + tmp_data[1];
  return id_value;
}

char HDC1000_Init() {
  GPIO_Set_Pin_Mode(&GPIOD_BASE, _GPIO_PIN_11, _GPIO_CFG_MODE_INPUT | _GPIO_CFG_PULL_UP); //PD10 - Data Ready pin
  GPIO_Digital_Input (&GPIOB_BASE, _GPIO_PINMASK_7);
  GPIO_Digital_Output (&GPIOB_BASE, _GPIO_PINMASK_6);
  //I2C INITIALIZATION PB6=SCL; PB7=SDA
  //If bus is busy wait SDA high before initializing I2C module
  GPIOB_ODR.B6 = 1;
  while (GPIOB_IDR.B7 == 0) {
    GPIOB_ODR.B6 = 0;
    Delay_us(10);
    GPIOB_ODR.B6 = 1;
    Delay_us(10);
  }
  I2C1_Init_Advanced(400000, &_GPIO_MODULE_I2C1_PB67);
  delay_ms(200);
  sensor_setup();
  return 1;
}

//August-Roche-Magnus approximation.
//http://en.wikipedia.org/wiki/Clausius%E2%80%93Clapeyron_relation#Meteorology_and_climatology
//tested: http://andrew.rsmas.miami.edu/bmcnoldy/Humidity.html
void Calculate_Dewpoint() {
  dewpoint = 243.04*(log(humidity/100)+((17.625*temperature)/(243.04+temperature)))
            /(17.625-log(humidity/100)-((17.625*temperature)/(243.04+temperature)));
}