/*
 * Project name:
     HDC1000 click (Demonstration of mikroBUS HDC1000 click board)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
     20150326:
       - initial release (DM);
 * Description:
     This code demonstrates how to use HDC1000 click with EasyMX Pro v7 board.
     HDC1000 sensor measures temperature and relative humidity.
 * Test configuration:
     MCU:             STM32F107VC
                      http://www.st.com/st-web-ui/static/active/en/resource/technical/document/reference_manual/CD00171190.pdf
     Dev. Board:      EasyMx PRO v7 for STM32
                      http://www.mikroe.com/eng/products/view/852/easymx-pro-v7-for-stm32/
     Oscillator:      HS-PLL 72.0000 MHz
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/arm/
 * NOTES:
     - Place HDC1000 click board at the mikroBUS socket 1 on the EasyMx Pro v7 board.
     - Put EasyTFT in its socket, turn on backlight using the SW11.3
 */
 
#include "HDC1000_Defs.h"
#include "HDC1000_Driver.h"
#include "resources.h"

char txt[16];
extern float temperature;
extern float humidity;
extern float dewpoint;
unsigned int device_id;

// TFT module connections
unsigned int TFT_DataPort at GPIOE_ODR;
sbit TFT_RST at GPIOE_ODR.B8;
sbit TFT_RS at GPIOE_ODR.B12;
sbit TFT_CS at GPIOE_ODR.B15;
sbit TFT_RD at GPIOE_ODR.B10;
sbit TFT_WR at GPIOE_ODR.B11;
sbit TFT_BLED at GPIOE_ODR.B9;
// End TFT module connections

void Display_Init(){
  TFT_Init_ILI9341_8bit(320, 240);
  TFT_BLED = 1;
  TFT_Set_Pen(CL_WHITE, 1);
  TFT_Set_Brush(1,CL_WHITE,0,0,0,0);
  TFT_Set_Font(TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Pen(CL_Black, 1);
  TFT_Line(20, 220, 300, 220);
  TFT_LIne(20,  46, 300,  46);
  TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("HDC1000 Click", 85, 14);
  TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("EasyMx PRO v7 for STM32", 19, 223);
  TFT_Set_Font(&Verdana12x13_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("www.mikroe.com", 200, 223);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
  TFT_Image_Jpeg(210, 75, rain_cloud_glossy_jpg);
}

void Draw_screen() {
  TFT_Set_Font(&Tahoma15x16_Bold, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("Temperature:",20,70);
  TFT_Write_Text("Relative Humidity:",20,100);
  TFT_Write_Text("Dewpoint:",20,130);
}

void Display_Values() {
  TFT_Set_Pen(CL_WHITE, 1);
  //temperature
  sprintf(txt, "%2.1f C", temperature);
  TFT_Rectangle(115,70,210,90);
  TFT_Write_Text(txt,115,70);
  //humidity
  sprintf(txt, "%2.1f", humidity);
  TFT_Rectangle(140,100,210,120);
  TFT_Write_Text(txt,140,100);
  TFT_Write_Char(0x25,170,100);
  //dewpoint
  sprintf(txt, "%2.1f C", dewpoint);
  TFT_Rectangle(90,130,210,150);
  TFT_Write_Text(txt,90,130);
}

void main() {
  Display_Init();
  while(!HDC1000_Init()); //wait for initialization
  device_id = Read_DeviceID();
  if (device_id = ID_NUMBER) {
    Draw_Screen();
    while(1) {
      Read_Sensor();
      Calculate_Dewpoint();
      Display_Values();
      delay_ms(2000);
    }
  }

  else {
    TFT_Write_Text("Error",10,30);
    while(1);
  }
}